(in-package :cl-matrix-db)

(defmethod insert (dao-model)
  (postmodern:with-connection cl-matrix-config:+database+
    (postmodern:insert-dao dao-model)))

;; USER

(defclass user ()
  ((id :col-type string :initarg :id :reader id)
   (password-hash :col-type string :initarg :password-hash :accessor password-hash))
  (:metaclass postmodern:dao-class)
  (:keys id)
  (:table-name users))

(defun get-id (identity)
  (cl-matrix::log-info (intern (getf identity :type "")))
  (case (alex:make-keyword (getf identity :type ""))
    (:|m.id.user| (getf identity :user))
    (otherwise (error 'cl-matrix::forbidden-error))))

(defun get-user (identity)
  (postmodern:with-connection cl-matrix-config:+database+
    (postmodern:get-dao 'user (get-id identity))))

(defun make-user (id password-hash)
  (make-instance 'user :id id :password-hash password-hash))

;; DEVICE

(defclass device ()
  ((user-id :col-type string :col-references ((user id)) :initarg :user-id :reader user-id)
   (id :col-type string :initarg :id :accessor id)
   (access-token :col-type string :initarg access-token :accessor access-token)
   (refresh-token :col-type string :initarg refresh-token :accessor refresh-token))
  (:metaclass postmodern:dao-class)
  (:keys access-token)
  (:table-name devices))

(defun get-device (access-token)
  (postmodern:with-connection cl-matrix-config:+database+
    (postmodern:get-dao 'device access-token)))

;; UTILS

(defun create-tables ()
  (postmodern:with-connection cl-matrix-config:+database+
    (postmodern:execute (postmodern:dao-table-definition 'user))
    (postmodern:execute (postmodern:dao-table-definition 'device))))

(defun drop-tables ()
  (postmodern:with-connection cl-matrix-config:+database+
    (postmodern:execute "DROP TABLE users;")
    (postmodern:execute "DROP TABLE devices;")))

(defun seed ()
  (let ((harry-dubois (make-user "@harry:rcm.org" (cl-matrix::generate-hash-with-salt "martinaise"))))
    (insert harry-dubois)))
